@@ -26,6 +26,13 @@ enum cgroup_delete_flag {
 	 * Recursively delete all child groups.
 	 */
 	CGFLAG_DELETE_RECURSIVE	= 2,
+
+	/**
+	 * Delete the cgroup only if it is empty, i.e. it has no subgroups and
+	 * no processes inside. This flag cannot be used with
+	 * CGFLAG_DELETE_RECURSIVE.
+	 */
+	CGFLAG_DELETE_EMPTY_ONLY	= 4,
 };
 
 /**
@@ -109,6 +116,15 @@ struct cgroup;
  */
 struct cgroup_controller;
 
+/**
+ * Uninitialized file/directory permissions used for task/control files.
+ */
+#define NO_PERMS (-1U)
+
+/**
+ * Uninitialized UID/GID used for task/control files.
+ */
+#define NO_UID_GID (-1U)
 
 /**
  * Allocate new cgroup structure. This function itself does not create new
@@ -172,7 +188,8 @@ void cgroup_free_controllers(struct cgroup *cgroup);
  * Physically create a control group in kernel. The group is created in all
  * hierarchies, which cover controllers added by cgroup_add_controller().
  * All parameters set by cgroup_add_value_* functions are written.
- * The created groups has owner which was set by cgroup_set_uid_gid().
+ * The created groups has owner which was set by cgroup_set_uid_gid() and
+ * permissions set by cgroup_set_permissions.
  * @param cgroup
  * @param ignore_ownership When nozero, all errors are ignored when setting
  * 	owner of the group and/or its tasks file.
@@ -278,7 +295,8 @@ int cgroup_delete_cgroup_ext(struct cgroup *cgroup, int flags);
 int cgroup_get_cgroup(struct cgroup *cgroup);
 
 /**
- * Copy all controllers, parameters and their values. All existing controllers
+ * Copy all controllers, their parameters and values. Group name, permissions
+ * and ownerships are not coppied. All existing controllers
  * in the source group are discarded.
  *
  * @param dst Destination group.
@@ -336,6 +354,22 @@ int cgroup_set_uid_gid(struct cgroup *cgroup, uid_t tasks_uid, gid_t tasks_gid,
 int cgroup_get_uid_gid(struct cgroup *cgroup, uid_t *tasks_uid,
 		gid_t *tasks_gid, uid_t *control_uid, gid_t *control_gid);
 
+/**
+ * Stores given file permissions of the group's control and tasks files
+ * into the @c cgroup data structure. Use NO_PERMS if permissions shouldn't
+ * be changed or a value which applicable to chmod(2). Please note that
+ * the given permissions are masked with the file owner's permissions.
+ * For example if a control file has permissions 640 and control_fperm is
+ * 471 the result will be 460.
+ * @param cgroup
+ * @param control_dperm Directory permission for the group.
+ * @param control_fperm File permission for the control files.
+ * @param task_fperm File permissions for task file.
+ */
+void cgroup_set_permissions(struct cgroup *cgroup,
+		mode_t control_dperm, mode_t control_fperm,
+		mode_t task_fperm);
+
 /**
  * @}
  * @name Group parameters
@@ -531,6 +565,12 @@ int cgroup_get_procs(char *name, char *controller, pid_t **pids, int *size);
 int cg_chmod_recursive(struct cgroup *cgroup, mode_t dir_mode,
 	int dirm_change, mode_t file_mode, int filem_change);
 
+/**
+ *  Get the name of the cgroup from a given cgroup
+ *  @param cgroup The cgroup whose name is needed
+ */
+char *cgroup_get_cgroup_name(struct cgroup *cgroup);
+
 /**
  * @}
  * @}
